#*    dsgui - a graphical user interface for Datove schranky
#*    Copyright (C) 2009-2010  CZ.NIC, z.s.p.o. (http://www.nic.cz)
#*
#*    This library is free software; you can redistribute it and/or
#*    modify it under the terms of the GNU Library General Public
#*    License as published by the Free Software Foundation; either
#*    version 2 of the License, or (at your option) any later version.
#*
#*    This library is distributed in the hope that it will be useful,
#*    but WITHOUT ANY WARRANTY; without even the implied warranty of
#*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#*    Library General Public License for more details.
#*
#*    You should have received a copy of the GNU Library General Public
#*    License along with this library; if not, write to the Free
#*    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
class Credentials for storage of account related info resides here
"""

import datetime, os
from message_store import DictMessageStore, DsdbMessageStore

class Credentials(object):
  """stores credentials to an account"""
  
  def __init__(self, username, password=None, test_account=False):
    self.username = username
    self.password = password
    self.test_account = test_account
    
  def __str__(self):
    return "%s:%s" % (self.username, self.test_account) 
    
  def change_password(self, new_password):
    """returns true if password was really changed"""
    if new_password == self.password:
      return False
    else:
      self.password = new_password
      return True
    
  def get_id(self):
    return (self.username, self.test_account)
    


class Account(object):
  """stores info about an account"""
  
  def __init__(self, credentials, dbdir):
    self.credentials = credentials
    self.account_info = None
    self.password_expiration_date = None
    self.open_message_storage(dbdir)
    
  def __str__(self):
    acc_type = self.credentials.test_account and "Test" or "Normal"
    return "Account '%s' - %s" % (self.credentials.username, acc_type)

  def open_message_storage(self, dbdir):
    """if dbdir is None, in-memory database will be used"""
    if dbdir is None:
      self.messages = DsdbMessageStore(None)
    else:
      self.messages = DsdbMessageStore(self.db_filename(dbdir))
    
  def set_account_info(self, ai):
    self.account_info = ai
    
  def is_password_expiring(self, tolerance=5):
    """tolerance is in days or a datetime.timedelta"""
    if type(tolerance) == int:
      tolerance = datetime.timedelta(days=tolerance)
    if self.password_expiration_date:
      return self.password_expiration_date < datetime.datetime.now() + tolerance
    else:
      return None
    
  def has_password_expired(self):
    return self.is_password_expiring(0)
    
  def get_password_expiration_delta(self):
    return self.password_expiration_date - datetime.datetime.now()

  def store_to_shelf(self, shelf):
    key = self.shelf_key()
    shelf[key] = {"account_info":self.account_info,
                  "password_expiration_date":self.password_expiration_date,
                  }
    shelf.sync()
    
  def load_from_shelf(self, shelf):
    data = shelf.get(self.shelf_key(), None)
    if data:
      account_info = data.get("account_info", None)
      ped = data.get("password_expiration_date", None)
      if ped != self.password_expiration_date:
        self.password_expiration_date = ped
      self.set_account_info(account_info)

  def db_filename(self, dbdir):
    fname = self.shelf_key().replace("True","1").replace("False","0")
    return os.path.join(dbdir, fname+".db")
      
  def shelf_key(self):
    """returns a key to be used for shelf based on current credentials"""
    return Account._create_shelf_key(self.credentials.username, self.credentials.test_account)
  
  @classmethod
  def _create_shelf_key(cls, username, test_account):
    """returns a key to be used for shelf based on current credentials"""
    return "%s___%s" % (username, test_account)

